function [bita,sigma2,vbeta] = HNR(Y,X,section,endog)
% PURPOSE: computes Holtz-Eakin, Rose, Newey (1988) IV for panel data
%-----------------------------------------------------------------------------
% USAGE:  [res1,res2]  = GLS_pull(YY,XX,nlags) 
% where:  Y = data matrix of dependent (long form)
%         X = data matrix of independent (long form)
%         section: row indices grouped per cross section (Ti), column vector
%         endog: column vector 1= endogenous regressor 0=exogenous
%                nlags = scalar, number of lags in test
%-----------------------------------------------------------------------------
% RETURNS:
%        bita = coefficients
%        sigma2 = std. of error
%        vbeta = vcov variance of bita
% ----------------------------------------------------------------------------
%
% written by:
% A. Romeu, 2018
% University of Murcia 

  [ti, sects]=hist(section,unique(section));
  npanels = length(ti);
  totn = sum(ti);
  tt = max(ti);
  nreg = length(endog);
  
  % Loop across sections and build instruments -----------------------------
  YtildeAC = [];
  XtildeAC = [];
  Qxz=0;
  Qzy=0;
  Az=0;
  for i=1:npanels
    paneli = (section == sects(i));
    Yi = Y(paneli,:);
    Xi = X(paneli,:);
    Ytilde = Yi(2:ti(i),:)-Yi(1:ti(i)-1,:) ;
    Xtilde = Xi(2:ti(i),:)-Xi(1:ti(i)-1,:) ;
    YtildeAC = [ YtildeAC ; Ytilde ];
    XtildeAC = [ XtildeAC ; Xtilde ];
    % Build instrument matrix
    Zi = [];
    if ti(i)<tt
      relleno =zeros(ti(i)-1,tt-ti(i));
      else
      relleno=[];
    end
    
    for j=1:nreg
      if endog(j)        
        Zi = [ Zi [ [zeros(1,ti(i)-2) ; diag(Xi(1:ti(i)-2,j))]  relleno ] ];
        else
        Zi = [ Zi Xi(2:ti(i),j)-Xi(1:ti(i)-1,j) ];
      end
    end
    
    % Build weighting matrix
    Hi = diag(ones(ti(i)-1,1),0) + diag(-.5*ones(ti(i)-2,1),1) + diag(-.5*ones(ti(i)-2,1),-1);
    % Acumulate matrices
    Qxz = Qxz + Zi'*Xtilde;
    Qzy = Qzy + Zi'*Ytilde;
    Az = Az + Zi'*Hi*Zi;
  end
  Az = Az^(-1); % There was a previous error here
  
  % Compute beta and its variance ------------------------------------------
  bita = (Qxz'*Az*Qxz)^(-1)*(Qxz'*Az*Qzy);
  ehat = YtildeAC - XtildeAC*bita;
  sigma2 = sum(ehat.^2)/(totn-nreg); % There was a previous error here
  vbeta = sigma2*((Qxz'*Az*Qxz)^(-1)); 
  
end
